      program tfergas
      include 'implno.dek'
      include 'const.dek'

! exercises the cold electron fermi gas equation of state

! declare
      real*8 :: den,ye,pres,dpdd,ener,dedd

! formats
 110  format(1x,t9,a,t25,a)
 111  format(1x,a5,1p2e16.8)


! get the density and assume an lectron fraction ye
 10   write(6,*) 'give the density =>'
      read(5,*) den

      ye = 0.5d0

! do it
      call fergas(den,ye,pres,dpdd,ener,dedd)

! say what we got
      write(6,110) 'quantity','d/d(rho)'
      write(6,111) 'pres',pres,dpdd
      write(6,111) 'ener',ener,dedd
      write(6,*) 

      goto 10
      end program tfergas




      subroutine fergas(den,ye,pres,dpdd,ener,dedd)
      include 'implno.dek'
      include 'const.dek'

! the eos for a cold electron fermi gas and its derivative
! see, chandra chapter 10, eq. 19-20, cox & guili equation 24.157 , hansen and kawaler chapter 3.5

! declare the pass
      real*8 :: den, ye, pres, dpdd, ener, dedd

! local variables
      real*8 :: fac1,fac2,fac3,dfac1,dfac2,dfac3,fx,dfx, &
                gac1,gac2,dgac1,dgac2,gx,dgx, & 
                x,dxdd,dpdx,dedx,deni

      real*8 :: x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17

      real*8, parameter :: third      = 1.0d0/3.0d0, &
                           twothird   = 2.0d0 * third, &
                           fourthird  = 4.0d0 * third, &
                           threeeight = 3.0d0/(8.0d0 * pi)

      real*8, parameter :: c5  = 8.0d0/5.0d0, &
                           c7  = 4.0d0/7.0d0, &
                           c9  = 1.0d0/3.0d0, &
                           c11 = 5.0d0/22.0d0, &
                           c13 = 35.0d0/208.0d0, &
                           c15 = 21.0d0/160.0d0, &
                           c17 = 231.0d0/2176.0d0

      real*8, parameter :: b5  = 12.0d0/5.0d0, &
                           b7  = 3.0d0/7.0d0, &
                           b9  = 1.0d0/6.0d0, &
                           b11 = 15.0d0/176.0d0, &
                           b13 = 21.0d0/416.0d0, &
                           b15 = 21.0d0/640.0d0, &
                           b17 = 99.0d0/4352.0d0

      real*8, parameter :: compton  = h/(me*clight), &
                           compton3 = compton*compton*compton, &
                           xne_con  = threeeight * compton3 * avo, &
                           pcon     = pi * third * me*clight*clight/compton3

      if (den .le. 0.0) stop 'bad pass in routine fergas'

! go 
      deni = 1.0d0/den

      x    = (xne_con * den * ye)**third
      dxdd = third * x * deni

      x2   = x   * x
      x3   = x2  * x


! usual expressions lose precision via a subtraction for small x, use the series expansion instead
! experiments show x = 0.1 is a reasonable break point to ~ 8 significant figures

      if (x .gt. 0.1) then 

! f function
! fac3 is a way of writing arcsinh (x)
       fac1 = sqrt(1.0d0 + x2)
       fac2 = 2.0d0 * x2  - 3.0d0
       fac3 = 3.0d0 * log(x + fac1)
       fx   = x*fac1*fac2 + fac3

! f function derivatives
       dfac1 = x/fac1
       dfac2 = 4.0d0 * x
       dfac3 = 3.0d0 * (1.0d0 + dfac1)/(x + fac1)
       dfx   = (fac1*fac2 + x*dfac1*fac2 + x*fac1*dfac2 + dfac3)

! g function
       gac1  = 8.0d0 * x3
       gac2  = fac1 - 1.0d0
       gx    = gac1*gac2 - fx

! g function derivatives
       dgac1 = 24.0d0 * x2
       dgac2 = dfac1
       dgx   = dgac1*gac2 + gac1*dgac2 - dfx


! for small x, wolfram alpha expansion
      else

       x4   = x3  * x
       x5   = x4  * x
       x6   = x5  * x
       x7   = x6  * x
       x8   = x7  * x
       x9   = x8  * x
       x10  = x9  * x
       x11  = x10 * x
       x12  = x11 * x
       x13  = x12 * x
       x14  = x13 * x
       x15  = x14 * x
       x16  = x15 * x
       x17  = x16 * x

       fx  = c5*x5 - c7*x7 + c9*x9 - c11*x11 + c13*x13 - c15*x15 + c17*x17   
       dfx = 5.0d0*c5*x4 - 7.0d0*c7*x6 + 9.0d0*c9*x8 - 11.0d0*c11*x10 + 13.0d0*c13*x12 - 15.0d0*c15*x14 + 17.0d0*c17*x16

       gx  = b5*x5 - b7*x7 + b9*x9 - b11*x11 + b13*x13 - b15*x15 + b17*x17   
       dgx = 5.0d0*b5*x4 - 7.0d0*b7*x6 + 9.0d0*b9*x8 - 11.0d0*b11*x10 + 13.0d0*b13*x12 - 15.0d0*b15*x14 + 17.0d0*b17*x16
      end if


! pressure in erg/cm**3
      pres = pcon * fx

! pressure derivative with density
      dpdx  = pcon * dfx
      dpdd  = dpdx * dxdd

! internal energy in erg/cm**3
      ener  = pcon * gx

! energy derivative with density
      dedx  = pcon * dgx
      dedd  = dedx * dxdd

! convert energies into erg/g
      ener = ener * deni
      dedd = (dedd - ener) * deni

      return
      end subroutine fergas
